@@ -21,7 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <qregexp.h>
+#include <QRegularExpression>
 #include <qtextdocument.h>
 #include <qurlquery.h>
 
@@ -181,7 +181,8 @@ int BookmarksProtocol::sizeOfGroup( const KBookmarkGroup &folder, bool real )
 void BookmarksProtocol::get( const QUrl& url )
 {
   QString path = url.path();
-  QRegExp regexp("^/(background|icon)/([\\S]+)");
+  const QRegularExpression regexp(QStringLiteral("^/(background|icon)/([\\S]+)"));
+  QRegularExpressionMatch rmatch;
 
   if (path.isEmpty() || path == "/") {
     echoIndex();
@@ -191,8 +192,8 @@ void BookmarksProtocol::get( const QUrl& url )
   } else if (path == "/editbookmarks") {
     KToolInvocation::kdeinitExec("keditbookmarks");
     echoHead("bookmarks:/");
-  } else if (regexp.indexIn(path) >= 0) {
-    echoImage(regexp.cap(1), regexp.cap(2), QUrlQuery(url).queryItemValue("size"));
+  } else if (path.indexOf(regexp, 0, &rmatch) >= 0) {
+    echoImage(rmatch.captured(1), rmatch.captured(2), QUrlQuery(url).queryItemValue("size"));
   } else {
     echoHead();
     echo("<p class=\"message\">" + i18n("Wrong request: %1", url.toDisplayString().toHtmlEscaped()) + "</p>");
